#ifdef CONFIG_RTNET_RTL
#define __NO_VERSION__
#endif
/*
 *	Routines having to do with the 'struct sk_buff' memory handlers.
 *
 *	Authors:	Alan Cox <iiitac@pyr.swan.ac.uk>
 *			Florian La Roche <rzsfl@rz.uni-sb.de>
 *
 *	Version:	$Id: skbuff.c,v 1.4 2001/10/01 16:20:29 ds Exp $
 *
 *	Fixes:	
 *		Alan Cox	:	Fixed the worst of the load balancer bugs.
 *		Dave Platt	:	Interrupt stacking fix.
 *	Richard Kooijman	:	Timestamp fixes.
 *		Alan Cox	:	Changed buffer format.
 *		Alan Cox	:	destructor hook for AF_UNIX etc.
 *		Linus Torvalds	:	Better skb_clone.
 *		Alan Cox	:	Added skb_copy.
 *		Alan Cox	:	Added all the changed routines Linus
 *					only put in the headers
 *		Ray VanTassle	:	Fixed --skb->lock in free
 *		Alan Cox	:	skb_copy copy arp field
 *		Andi Kleen	:	slabified it.
 *
 *	NOTE:
 *		The __skb_ routines should be called with interrupts 
 *	disabled, or you better be *real* sure that the operation is atomic 
 *	with respect to whatever list is being frobbed (e.g. via lock_sock()
 *	or via disabling bottom half handlers, etc).
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version
 *	2 of the License, or (at your option) any later version.
 */

/*
 *	The functions in this file will not compile correctly with gcc 2.4.x
 */

#include <linux/version.h>
#include <linux/config.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/mm.h>
#include <linux/interrupt.h>
#include <linux/in.h>
#include <linux/inet.h>
#include <linux/malloc.h>
#include <linux/netdevice.h>
#include <linux/string.h>
#include <linux/skbuff.h>
#include <linux/slab.h>
#include <linux/init.h>

#include <net/ip.h>
#include <net/protocol.h>
#include <net/dst.h>
#include <net/tcp.h>
#include <net/udp.h>
#include <net/sock.h>
#include <rtnet/rtnet.h>

#include <asm/uaccess.h>
#include <asm/system.h>

/*	rt skbuff routines
 *  
 *  The goal for RT skbuff allocation is to have an O(1) allocator and
 *  deallocator.  This means wasting a *lot* of memory.
 *
 *  The rtskbuff cache is a sk_buff list that we allocated at boot.
 *  When we need a skbuff, we take the first on the list.  When it
 *  is freed, we simply add it to the front of the list.
 */

#define RT

struct sk_buff_head rtskb_free;
#define rtskb_free_list (&rtskb_free)

#define RTSKB_LEN 1000
#define RTSKB_MAXSIZE 1552

static atomic_t lowwater = ATOMIC_INIT(0);


/* 	Allocate a new skbuff. We do this ourselves so we can fill in a few
 *	'private' fields and also do memory statistics to find all the
 *	[BEEP] leaks.
 * 
 */

struct sk_buff *alloc_rtskb(unsigned int size,int gfp_mask)
{
	struct sk_buff *skb;
	u8 *data;

	if(size>RTSKB_MAXSIZE)
		return NULL;

	skb=__skb_dequeue(rtskb_free_list);

	if (skb == NULL) {
		panic("rtnet OOM\n");

		return NULL;
	}

#if 0
rt_printk("alloc_rtskb(): %p\n",skb);
#endif

	/* XXX is this right? */
	skb->truesize = size;

	/* skb->head is where the kmalloc pointer is kept */
	data=skb->head;

	/* Load the data pointers. */
	skb->data = data;
	skb->tail = data;
	skb->end = data + size;

	/* Set up other state */
	skb->len = 0;	/* XXX eh? */
	skb->is_clone = 0;
	skb->cloned = 0;

	/* XXX */
#if 1
	atomic_set(&skb->users, 1); 
	atomic_set(skb_datarefp(skb), 1);
#else
	skb->users=1; 
	*(skb_datarefp(skb))=1;
#endif

	return skb;

}


/*
 *	Slab constructor for a skb head. 
 */ 
static inline void skb_headerinit(void *p, kmem_cache_t *cache, 
				  unsigned long flags)
{
	struct sk_buff *skb = p;

	skb->destructor = NULL;
	skb->pkt_type = PACKET_HOST;	/* Default type */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,0)
	skb->pkt_bridged = 0;		/* Not bridged */
#endif
	skb->prev = skb->next = NULL;
	skb->list = NULL;
	skb->sk = NULL;
	skb->stamp.tv_sec=0;	/* No idea about time */
	skb->ip_summed = 0;
	skb->security = 0;	/* By default packets are insecure */
	skb->dst = NULL;
#if 0 && defined(CONFIG_IP_FIREWALL)
        skb->fwmark = 0;
#endif
	memset(skb->cb, 0, sizeof(skb->cb));
	skb->priority = 0;
}

#ifndef RT /* XXX don't need */
/*
 *	Free an skbuff by memory without cleaning the state. 
 */
void kfree_skbmem(struct sk_buff *skb)
{
	if (!skb->cloned || atomic_dec_and_test(skb_datarefp(skb)))  
		kfree(skb->head);

	kmem_cache_free(skbuff_head_cache, skb);
	atomic_dec(&net_skbcount);
}
#endif

/*
 *	Free an sk_buff. Release anything attached to the buffer. Clean the state.
 */

void __kfree_rtskb(struct sk_buff *skb)
{
	if(!skb){
		rt_printk("kfree_rtskb() skb=NULL\n");
		return;
	}
	if (skb->list){
	 	rt_printk(KERN_WARNING "kfree_rtskb() passed an skb still "
		       "on a list\n");
		return;
	}

#if 0
	dst_release(skb->dst); /* XXX */
#endif
	if(skb->destructor)
		skb->destructor(skb); /* XXX */
	skb_headerinit(skb, NULL, 0);  /* clean state */

	__skb_queue_head(rtskb_free_list,skb);
}

/*
 *	Duplicate an sk_buff. The new one is not owned by a socket.
 */

struct sk_buff *rtskb_clone(struct sk_buff *skb, int gfp_mask)
{
	rt_printk("Fix rtskb_clone()!\n");

	return rtskb_copy(skb,gfp_mask);

	/* XXX how to do this? */
#if 0
	struct sk_buff *n;
	
	n = kmem_cache_alloc(skbuff_head_cache, gfp_mask);
	if (!n)
		return NULL;

	memcpy(n, skb, sizeof(*n));
	atomic_inc(skb_datarefp(skb));
	skb->cloned = 1;
       
	atomic_inc(&net_allocs);
	atomic_inc(&net_skbcount);
	dst_clone(n->dst);
	n->cloned = 1;
	n->next = n->prev = NULL;
	n->list = NULL;
	n->sk = NULL;
	n->is_clone = 1;
	atomic_set(&n->users, 1);
	n->destructor = NULL;
	return n;
#endif
}

/*
 *	This is slower, and copies the whole data area 
 */
 
struct sk_buff *rtskb_copy(struct sk_buff *skb, int gfp_mask)
{
	struct sk_buff *n;
	unsigned long offset;

	/*
	 *	Allocate the copy buffer
	 */
	 
	n=alloc_rtskb(skb->end - skb->head, gfp_mask);
	if(n==NULL)
		return NULL;

	/*
	 *	Shift between the two data areas in bytes
	 */
	 
	offset=n->head-skb->head;

	/* Set the data pointer */
	skb_reserve(n,skb->data-skb->head);
	/* Set the tail pointer and length */
	skb_put(n,skb->len);
	/* Copy the bytes */
	memcpy(n->head,skb->head,skb->end-skb->head);
	n->csum = skb->csum;
	n->list=NULL;
	n->sk=NULL;
	n->dev=skb->dev;
	n->priority=skb->priority;
	n->protocol=skb->protocol;
	n->dst=dst_clone(skb->dst);
	n->h.raw=skb->h.raw+offset;
	n->nh.raw=skb->nh.raw+offset;
	n->mac.raw=skb->mac.raw+offset;
	memcpy(n->cb, skb->cb, sizeof(skb->cb));
	n->used=skb->used;
	n->is_clone=0;
	atomic_set(&n->users, 1);
	n->pkt_type=skb->pkt_type;
	n->stamp=skb->stamp;
	n->destructor = NULL;
	n->security=skb->security;
#ifdef CONFIG_IP_FIREWALL
        n->fwmark = skb->fwmark;
#endif
	return n;
}

#ifndef RT
struct sk_buff *skb_realloc_headroom(struct sk_buff *skb, int newheadroom)
{
	struct sk_buff *n;
	unsigned long offset;
	int headroom = skb_headroom(skb);

	/*
	 *	Allocate the copy buffer
	 */
 	 
	n=alloc_skb(skb->truesize+newheadroom-headroom, GFP_ATOMIC);
	if(n==NULL)
		return NULL;

	skb_reserve(n,newheadroom);

	/*
	 *	Shift between the two data areas in bytes
	 */
	 
	offset=n->data-skb->data;

	/* Set the tail pointer and length */
	skb_put(n,skb->len);
	/* Copy the bytes */
	memcpy(n->data,skb->data,skb->len);
	n->list=NULL;
	n->sk=NULL;
	n->priority=skb->priority;
	n->protocol=skb->protocol;
	n->dev=skb->dev;
	n->dst=dst_clone(skb->dst);
	n->h.raw=skb->h.raw+offset;
	n->nh.raw=skb->nh.raw+offset;
	n->mac.raw=skb->mac.raw+offset;
	memcpy(n->cb, skb->cb, sizeof(skb->cb));
	n->used=skb->used;
	n->is_clone=0;
	atomic_set(&n->users, 1);
	n->pkt_type=skb->pkt_type;
	n->stamp=skb->stamp;
	n->destructor = NULL;
	n->security=skb->security;
#ifdef CONFIG_IP_FIREWALL
        n->fwmark = skb->fwmark;
#endif

	return n;
}
#endif




void __init rtskb_init(void)
{
	int i;
	struct sk_buff *skb;

	skb_queue_head_init(rtskb_free_list);

	for(i=0;i<RTSKB_LEN;i++){
		skb=alloc_skb(RTSKB_MAXSIZE,GFP_KERNEL);
		if (!skb)
			panic("cannot create rtskbuff cache");
		__skb_queue_tail(rtskb_free_list,skb);
	}

	atomic_set(&lowwater,skb_queue_len(rtskb_free_list));
}

