#ifdef CONFIG_RTNET_RTL
#define __NO_VERSION__
#endif
/*
 * INET		An implementation of the TCP/IP protocol suite for the LINUX
 *		operating system.  INET is implemented using the  BSD Socket
 *		interface as the means of communication with the user level.
 *
 *		ROUTE - implementation of the IP router.
 *
 * Version:	$Id: route.c,v 1.2 2000/07/05 16:23:31 ds Exp $
 *
 * Authors:	Ross Biro, <bir7@leland.Stanford.Edu>
 *		Fred N. van Kempen, <waltje@uWalt.NL.Mugnet.ORG>
 *		Alan Cox, <gw4pts@gw4pts.ampr.org>
 *		Linus Torvalds, <Linus.Torvalds@helsinki.fi>
 *		Alexey Kuznetsov, <kuznet@ms2.inr.ac.ru>
 *
 * Fixes:
 *		Alan Cox	:	Verify area fixes.
 *		Alan Cox	:	cli() protects routing changes
 *		Rui Oliveira	:	ICMP routing table updates
 *		(rco@di.uminho.pt)	Routing table insertion and update
 *		Linus Torvalds	:	Rewrote bits to be sensible
 *		Alan Cox	:	Added BSD route gw semantics
 *		Alan Cox	:	Super /proc >4K 
 *		Alan Cox	:	MTU in route table
 *		Alan Cox	: 	MSS actually. Also added the window
 *					clamper.
 *		Sam Lantinga	:	Fixed route matching in rt_del()
 *		Alan Cox	:	Routing cache support.
 *		Alan Cox	:	Removed compatibility cruft.
 *		Alan Cox	:	RTF_REJECT support.
 *		Alan Cox	:	TCP irtt support.
 *		Jonathan Naylor	:	Added Metric support.
 *	Miquel van Smoorenburg	:	BSD API fixes.
 *	Miquel van Smoorenburg	:	Metrics.
 *		Alan Cox	:	Use __u32 properly
 *		Alan Cox	:	Aligned routing errors more closely with BSD
 *					our system is still very different.
 *		Alan Cox	:	Faster /proc handling
 *	Alexey Kuznetsov	:	Massive rework to support tree based routing,
 *					routing caches and better behaviour.
 *		
 *		Olaf Erb	:	irtt wasn't being copied right.
 *		Bjorn Ekwall	:	Kerneld route support.
 *		Alan Cox	:	Multicast fixed (I hope)
 * 		Pavel Krauz	:	Limited broadcast fixed
 *		Mike McLagan	:	Routing by source
 *	Alexey Kuznetsov	:	End of old history. Splitted to fib.c and
 *					route.c and rewritten from scratch.
 *		Andi Kleen	:	Load-limit warning messages.
 *	Vitaly E. Lavrov	:	Transparent proxy revived after year coma.
 *	Vitaly E. Lavrov	:	Race condition in ip_route_input_slow.
 *	Tobias Ringstrom	:	Uninitialized res.type in ip_route_output_slow.
 *	Vladimir V. Ivanov	:	IP rule info (flowid) is really useful.
 *		Marc Boucher	:	routing by fwmark
 *
 *		This program is free software; you can redistribute it and/or
 *		modify it under the terms of the GNU General Public License
 *		as published by the Free Software Foundation; either version
 *		2 of the License, or (at your option) any later version.
 */

#include <linux/config.h>
#include <asm/uaccess.h>
#include <asm/system.h>
#include <asm/bitops.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/mm.h>
#include <linux/string.h>
#include <linux/socket.h>
#include <linux/sockios.h>
#include <linux/errno.h>
#include <linux/in.h>
#include <linux/inet.h>
#include <linux/netdevice.h>
#include <linux/proc_fs.h>
#include <linux/init.h>
#include <linux/skbuff.h>
#include <linux/rtnetlink.h>
#include <linux/inetdevice.h>
#include <linux/igmp.h>
#include <linux/pkt_sched.h>
#include <linux/mroute.h>
#include <net/protocol.h>
#include <net/ip.h>
#include <net/route.h>
#include <net/sock.h>
#include <net/ip_fib.h>
#include <net/arp.h>
#include <net/tcp.h>
#include <net/icmp.h>
#ifdef CONFIG_SYSCTL
#include <linux/sysctl.h>
#endif
#include <rtnet/rtnet.h>

#define RT

#define N_RT_RTABLES 50

static struct rt_rtable *rt_rtable_free_list;
static struct rt_rtable rt_rtable_cache[N_RT_RTABLES];

static struct rt_rtable *rt_rtables;
static struct rt_rtable *rt_rtables_generic;


static struct rt_rtable *rt_alloc(void)
{
	struct rt_rtable *rt;

	rt=rt_rtable_free_list;
	if(rt){
		rt_rtable_free_list=rt->next;

		rt->use_count=1;
		rt->next=NULL;
	}else{
		rt_printk("no more routes\n");
	}

	return rt;
}



struct rt_rtable *rt_ip_route_add(struct rt_device *rtdev,u32 addr,u32 mask)
{
	struct rt_rtable *rt;

	rt=rt_alloc();
	if(!rt)
		return rt;

	rt->rt_dst=addr;
	rt->rt_dst_mask=mask;

	if(!rtdev){
		/* find appropriate dev (maybe?) */
	}
	rt->rt_dev=rtdev;

	rt->rt_src=rtdev->local_addr;

	rt->next=rt_rtables_generic;
	rt_rtables_generic=rt;

	return rt;
}

struct rt_rtable *rt_ip_route_find(u32 daddr)
{
	struct rt_rtable *rt,*new_rt;

	for(rt=rt_rtables;rt;rt=rt->next){
		if(rt->rt_dst==daddr)
			return rt;
	}

	for(rt=rt_rtables_generic;rt;rt=rt->next){
		if(rt->rt_dst==(daddr&rt->rt_dst_mask))
			goto found;
	}
	return NULL;

found:
	new_rt=rt_alloc();
	
	new_rt->rt_dev=rt->rt_dev;
	new_rt->rt_src=rt->rt_src;

	new_rt->rt_dst=daddr;
	new_rt->rt_dst_mask=0xffffffff;

	memset(new_rt->rt_dst_mac_addr,0,6);

	return rt;
}



int rt_ip_route_input(struct sk_buff *skb,u32 daddr,u32 saddr,
		u8 tos,struct rt_device *dev)
{
	struct rt_rtable *rt;

	/*  careful... the packet is incoming, but the routing
	 *  tables are always outgoing.  This is different from
	 *  normal Linux.
	 */
	for(rt=rt_rtables;rt!=NULL;rt=rt->next){
		if(rt->rt_dst==saddr && rt->rt_src==daddr &&
		   rt->rt_dev==dev){
			/* matches a specific route */
			skb->dst=(void *)rt;
			return 0;
		}
	}

	for(rt=rt_rtables_generic;rt!=NULL;rt=rt->next){
		if(rt->rt_dst==(saddr&rt->rt_dst_mask) &&
		   rt->rt_src==daddr &&
		   rt->rt_dev==dev){
			/* matches a generic route */
			skb->dst=(void *)rt;
			goto route;
		}
	}

//rt_printk("rt_ip_route_input() dropping saddr=%08x daddr=%08x dev=%s\n",saddr,daddr,dev->dev->name);

	skb->dst = NULL;

	return -EHOSTUNREACH;

route:
	/* XXX */
rt_printk("rt_ip_route_input() adding route saddr=%08x daddr=%08x dev=%s\n",saddr,daddr,dev->dev->name);
	rt=rt_alloc();

	rt->rt_dst=saddr;
	rt->rt_src=daddr;
	rt->rt_dev=dev;
	rt->rt_iif=dev->dev->ifindex;
	memcpy(rt->rt_dst_mac_addr,skb->mac.ethernet->h_source,6);

	rt->next=rt_rtables;
	rt_rtables=rt;

	skb->dst=(void *)rt;

	return 0;
}

static struct rt_device *rt_ip_dev_find(u32 saddr)
{
	struct rt_device *d=rt_devices;

	for(d=rt_devices;d!=NULL;d=d->next){
		if(!saddr || saddr==d->local_addr)
			return d;
	}

	rt_printk("ack! rt_ip_dev_find() returning NULL\n");

	return NULL;
}

int rt_ip_route_output(struct rt_rtable **rp, u32 daddr, u32 saddr, u32 tos, int oif)
{
	struct rt_rtable *rt;

#if 0
rt_printk("route find: daddr=%08x saddr=%08x oif=%d\n",daddr,saddr,oif);
#endif
	for(rt=rt_rtables;rt;rt=rt->next){
#if 0
rt_printk("route: daddr=%08x saddr=%08x oif=%d\n",rt->rt_dst,rt->rt_src,rt->rt_oif);
#endif
		if(rt->rt_dst==daddr && (!saddr || rt->rt_src==saddr) &&
		   (!oif || rt->rt_oif==oif)){
			*rp=rt;
			return 0;
		}
	}

	/*
	 * A normal network stack would hold a packet and wait for an ARP, when a
	 * route can't be found.  We return an EHOSTUNREACH
	 *
	 * One alternative is to broadcast the packet...
	 */

rt_printk("rt_ip_route_output() route not found daddr=%08x saddr=%08x oif=%d\n",daddr,saddr,oif);

	return -EHOSTUNREACH;

#if 0
	/* XXX */
	rt=rt_alloc();

	rt->rt_dst=daddr;
	rt->rt_src=saddr;
	rt->rt_oif=oif;
	if(!oif){
		rt->rt_dev = rt_ip_dev_find(saddr);
	}else{
		rt->rt_dev = rt_dev_get_by_index(oif);
	}
	rt->rt_iif=rt->rt_dev->dev->ifindex;
/* XXX fix */
if(!saddr){
	rt->rt_src=rt->rt_dev->local_addr;
}

	rt->next=rt_rtables;
	rt_rtables=rt;
	*rp=rt;
#endif

	return 0;
}

__initfunc(void rt_ip_rt_init(void))
{
	int i;

	for(i=0;i<N_RT_RTABLES-1;i++){
		(rt_rtable_cache+i)->next=rt_rtable_cache+i+1;
	}
	rt_rtable_free_list=rt_rtable_cache;
}
