#ifdef CONFIG_RTNET_RTL
#define __NO_VERSION__
#endif
/*
 * INET		An implementation of the TCP/IP protocol suite for the LINUX
 *		operating system.  INET is implemented using the  BSD Socket
 *		interface as the means of communication with the user level.
 *
 *		The User Datagram Protocol (UDP).
 *
 * Version:	$Id: udp.c,v 1.6 2001/10/01 16:26:58 ds Exp $
 *
 * Authors:	Ross Biro, <bir7@leland.Stanford.Edu>
 *		Fred N. van Kempen, <waltje@uWalt.NL.Mugnet.ORG>
 *		Arnt Gulbrandsen, <agulbra@nvg.unit.no>
 *		Alan Cox, <Alan.Cox@linux.org>
 *
 * Fixes:
 *		Alan Cox	:	verify_area() calls
 *		Alan Cox	: 	stopped close while in use off icmp
 *					messages. Not a fix but a botch that
 *					for udp at least is 'valid'.
 *		Alan Cox	:	Fixed icmp handling properly
 *		Alan Cox	: 	Correct error for oversized datagrams
 *		Alan Cox	:	Tidied select() semantics. 
 *		Alan Cox	:	udp_err() fixed properly, also now 
 *					select and read wake correctly on errors
 *		Alan Cox	:	udp_send verify_area moved to avoid mem leak
 *		Alan Cox	:	UDP can count its memory
 *		Alan Cox	:	send to an unknown connection causes
 *					an ECONNREFUSED off the icmp, but
 *					does NOT close.
 *		Alan Cox	:	Switched to new sk_buff handlers. No more backlog!
 *		Alan Cox	:	Using generic datagram code. Even smaller and the PEEK
 *					bug no longer crashes it.
 *		Fred Van Kempen	: 	Net2e support for sk->broadcast.
 *		Alan Cox	:	Uses skb_free_datagram
 *		Alan Cox	:	Added get/set sockopt support.
 *		Alan Cox	:	Broadcasting without option set returns EACCES.
 *		Alan Cox	:	No wakeup calls. Instead we now use the callbacks.
 *		Alan Cox	:	Use ip_tos and ip_ttl
 *		Alan Cox	:	SNMP Mibs
 *		Alan Cox	:	MSG_DONTROUTE, and 0.0.0.0 support.
 *		Matt Dillon	:	UDP length checks.
 *		Alan Cox	:	Smarter af_inet used properly.
 *		Alan Cox	:	Use new kernel side addressing.
 *		Alan Cox	:	Incorrect return on truncated datagram receive.
 *	Arnt Gulbrandsen 	:	New udp_send and stuff
 *		Alan Cox	:	Cache last socket
 *		Alan Cox	:	Route cache
 *		Jon Peatfield	:	Minor efficiency fix to sendto().
 *		Mike Shaver	:	RFC1122 checks.
 *		Alan Cox	:	Nonblocking error fix.
 *	Willy Konynenberg	:	Transparent proxying support.
 *		Mike McLagan	:	Routing by source
 *		David S. Miller	:	New socket lookup architecture.
 *					Last socket cache retained as it
 *					does have a high hit rate.
 *		Olaf Kirch	:	Don't linearise iovec on sendmsg.
 *		Andi Kleen	:	Some cleanups, cache destination entry
 *					for connect. 
 *	Vitaly E. Lavrov	:	Transparent proxy revived after year coma.
 *		Melvin Smith	:	Check msg_name not msg_namelen in sendto(),
 *					return ENOTCONN for unconnected sockets (POSIX)
 *		Janos Farkas	:	don't deliver multi/broadcasts to a different
 *					bound-to-device socket
 *
 *
 *		This program is free software; you can redistribute it and/or
 *		modify it under the terms of the GNU General Public License
 *		as published by the Free Software Foundation; either version
 *		2 of the License, or (at your option) any later version.
 */
 
/* RFC1122 Status:
   4.1.3.1 (Ports):
     SHOULD send ICMP_PORT_UNREACHABLE in response to datagrams to 
       an un-listened port. (OK)
   4.1.3.2 (IP Options)
     MUST pass IP options from IP -> application (OK)
     MUST allow application to specify IP options (OK)
   4.1.3.3 (ICMP Messages)
     MUST pass ICMP error messages to application (OK -- except when SO_BSDCOMPAT is set)
   4.1.3.4 (UDP Checksums)
     MUST provide facility for checksumming (OK)
     MAY allow application to control checksumming (OK)
     MUST default to checksumming on (OK)
     MUST discard silently datagrams with bad csums (OK, except during debugging)
   4.1.3.5 (UDP Multihoming)
     MUST allow application to specify source address (OK)
     SHOULD be able to communicate the chosen src addr up to application
       when application doesn't choose (DOES - use recvmsg cmsgs)
   4.1.3.6 (Invalid Addresses)
     MUST discard invalid source addresses (OK -- done in the new routing code)
     MUST only send datagrams with one of our addresses (OK)
*/

#include <linux/types.h>
#include <linux/fcntl.h>
#include <linux/socket.h>
#include <linux/sockios.h>
#include <linux/in.h>
#include <linux/errno.h>
#include <linux/timer.h>
#include <linux/mm.h>
#include <linux/config.h>
#include <linux/inet.h>
#include <linux/netdevice.h>
#include <net/snmp.h>
#include <net/ip.h>
#include <net/protocol.h>
#include <linux/skbuff.h>
#include <net/sock.h>
#include <net/udp.h>
#include <net/icmp.h>
#include <net/route.h>
#include <net/checksum.h>
#include <linux/init.h>
#include <rtnet/rtnet.h>

#define RT

static struct rtsocket *udp_sockets;
struct rtsocket_ops rt_udp_socket_ops;

static inline int rt_udp_lport_inuse(u16 num)
{
	struct rtsocket *sk;

	for(sk=udp_sockets; sk != NULL; sk = sk->next) {
		if(sk->num == num)
			return 1;
	}
	return 0;
}

static unsigned short goodsocknum=1024;

unsigned short rt_udp_good_socknum(void)
{
	unsigned short goodsock;

	do{
		goodsock=goodsocknum++;
	}while(rt_udp_lport_inuse(goodsock));

	return goodsock;
}


struct rtsocket *rt_udp_v4_lookup(u32 saddr, u16 sport, u32 daddr, u16 dport, int dif)
{
	struct rtsocket *sk, *result = NULL;
	unsigned short hnum = ntohs(dport);
	int badness = -1;

	for(sk = udp_sockets; sk != NULL; sk = sk->next){
		if((sk->num == hnum) && !(sk->dead && (sk->state == TCP_CLOSE))) {
			int score = 0;
			if(sk->rcv_saddr) {
				if(sk->rcv_saddr != daddr)
					continue;
				score++;
			}
			if(sk->daddr) {
				if(sk->daddr != saddr)
					continue;
				score++;
			}
			if(sk->dport) {
				if(sk->dport != sport)
					continue;
				score++;
			}
#if 0
			if(sk->bound_dev_if) {
				if(sk->bound_dev_if != dif)
					continue;
				score++;
			}
#endif
			if(score == 3) {
				result = sk;
				break;
			} else if(score > badness) {
				result = sk;
				badness = score;
			}
		}
	}
	return result;
}


static unsigned short udp_check(struct udphdr *uh, int len, unsigned long saddr, unsigned long daddr, unsigned long base)
{
	return(csum_tcpudp_magic(saddr, daddr, len, IPPROTO_UDP, base));
}

struct udpfakehdr 
{
	struct udphdr uh;
	u32 saddr;
	u32 daddr;
	struct iovec *iov;
	u32 wcheck;
};

 
static int rt_udp_getfrag(const void *p, char * to, unsigned int offset, unsigned int fraglen) 
{
	struct udpfakehdr *ufh = (struct udpfakehdr *)p;
	if (offset==0) {
		memcpy_fromkerneliovec(to+sizeof(struct udphdr), ufh->iov,
			fraglen-sizeof(struct udphdr));
 		ufh->wcheck = csum_partial(to+sizeof(struct udphdr), fraglen-sizeof(struct udphdr),
					   ufh->wcheck);
 		ufh->wcheck = csum_partial((char *)ufh, sizeof(struct udphdr),
					   ufh->wcheck);
		ufh->uh.check = csum_tcpudp_magic(ufh->saddr, ufh->daddr, 
					  ntohs(ufh->uh.len),
					  IPPROTO_UDP, ufh->wcheck);
		if (ufh->uh.check == 0)
			ufh->uh.check = -1;
		memcpy(to, ufh, sizeof(struct udphdr));
		return 0;
	}
printk("rt_udp_getfrag: unimplemented case\n");
#if 0
	memcpy_fromkerneliovec(to, ufh->iov, offset-sizeof(struct udphdr),
			fraglen);
	csum_partial(to, ufh->iov, fraglen, &ufh->wcheck);
#endif

	return 0;
}


int rt_udp_sendmsg(struct rtsocket *sk, const struct msghdr *msg, int len)
{
	int ulen = len + sizeof(struct udphdr);
	struct ipcm_cookie ipc;
	struct udpfakehdr ufh;
	struct rt_rtable *rt = NULL;
	int connected = 0;
	u32 daddr;
	u32 saddr;
	int err;

	/* This check is ONLY to check for arithmetic overflow
	   on integer(!) len. Not more! Real check will be made
	   in ip_build_xmit --ANK

	   BTW socket.c -> af_*.c -> ... make multiple
	   invalid conversions size_t -> int. We MUST repair it f.e.
	   by replacing all of them with size_t and revise all
	   the places sort of len += sizeof(struct iphdr)
	   If len was ULONG_MAX-10 it would be cathastrophe  --ANK
	 */

	if (len < 0 || len > 0xFFFF)
		return -EMSGSIZE;

	/* 
	 *	Check the flags.
	 */

	if (msg->msg_flags&MSG_OOB)	/* Mirror BSD error message compatibility */
		return -EOPNOTSUPP;

	if (msg->msg_flags&~(MSG_DONTROUTE|MSG_DONTWAIT|MSG_NOSIGNAL))
	  	return -EINVAL;

	/*
	 *	Get and verify the address. 
	 */
	 
	if (msg->msg_name) {
		struct sockaddr_in * usin = (struct sockaddr_in*)msg->msg_name;
		if (msg->msg_namelen < sizeof(*usin))
			return(-EINVAL);
		if (usin->sin_family != AF_INET) {
			static int complained;
			if (!complained++)
				rt_printk(KERN_WARNING "%s forgot to set AF_INET in udp sendmsg. Fix it!\n", current->comm);
			if (usin->sin_family)
				return -EINVAL;
		}
		ufh.daddr = usin->sin_addr.s_addr;
		ufh.uh.dest = usin->sin_port;
		if (ufh.uh.dest == 0)
			return -EINVAL;
	} else {
		if (sk->state != TCP_ESTABLISHED)
			return -ENOTCONN;
		ufh.daddr = sk->daddr;
		ufh.uh.dest = sk->dport;
		/* Open fast path for connected socket.
		   Route will not be used, if at least one option is set.
		 */
		connected = 1;
  	}
	/* XXX hack alert */
	saddr = sk->saddr;
	/* XXX hack alert */
	memset(&ipc,0,sizeof(ipc));
#if 0
	ipc.addr = sk->saddr;
#endif
	ufh.uh.source = sk->sport;

#if 0
	ipc.opt = NULL;
	ipc.oif = sk->bound_dev_if;
	if (msg->msg_controllen) {
		err = ip_cmsg_send(msg, &ipc);
		if (err)
			return err;
		if (ipc.opt)
			free = 1;
		connected = 0;
	}
	if (!ipc.opt)
		ipc.opt = sk->opt;

	ufh.saddr = ipc.addr;
	ipc.addr = daddr = ufh.daddr;

	if (ipc.opt && ipc.opt->srr) {
		if (!daddr)
			return -EINVAL;
		daddr = ipc.opt->faddr;
		connected = 0;
	}
#endif
	daddr = ufh.daddr;
#if 0
	if (sk->localroute || (msg->msg_flags&MSG_DONTROUTE) || 
	    (ipc.opt && ipc.opt->is_strictroute)) {
		tos |= RTO_ONLINK;
		connected = 0;
	}
#endif

#if 0
	if (MULTICAST(daddr)) {
		if (!ipc.oif)
			ipc.oif = sk->ip_mc_index;
		if (!ufh.saddr)
			ufh.saddr = sk->ip_mc_addr;
		connected = 0;
	}
#endif

#if 0
	if (connected)
		rt = (struct rtable*)dst_clone(sk->dst_cache);

	if (rt == NULL) {
		err = ip_route_output(&rt, daddr, ufh.saddr,
			 tos, ipc.oif);
		if (err) 
			goto out;

		err = -EACCES;
		if (rt->rt_flags&RTCF_BROADCAST && !sk->broadcast) 
			goto out;
	}
#endif
	err=rt_ip_route_output(&rt, daddr, saddr, RT_TOS(sk->ip_tos),0);
	if(err)
		goto out;

	ufh.saddr = rt->rt_src;
#if 0
	if (!ipc.addr)
		ufh.daddr = ipc.addr = rt->rt_dst;
#endif
	ufh.daddr = rt->rt_dst;
	ufh.uh.len = htons(ulen);
	ufh.uh.check = 0;
	ufh.iov = msg->msg_iov;
	ufh.wcheck = 0;

	/* RFC1122: OK.  Provides the checksumming facility (MUST) as per */
	/* 4.1.3.4. It's configurable by the application via setsockopt() */
	/* (MAY) and it defaults to on (MUST). */

	{ /* hack alert */
	mm_segment_t oldfs = get_fs(); set_fs(KERNEL_DS);
	err = rt_ip_build_xmit(sk,rt_udp_getfrag,
			    &ufh, ulen, &ipc, rt, msg->msg_flags);
	set_fs(oldfs);
	}

out:
#if 0
	ip_rt_put(rt);
	if (free)
		kfree(ipc.opt);
#endif
	if (!err) {
#if 0
		udp_statistics.UdpOutDatagrams++;
#endif
		return len;
	}
	return err;
}

#if 0
/*
 *	IOCTL requests applicable to the UDP protocol
 */
 
int udp_ioctl(struct sock *sk, int cmd, unsigned long arg)
{
	return(-ENOIOCTLCMD);
}
#endif


int rt_udp_connect(struct rtsocket *sk, struct sockaddr *uaddr, int addr_len)
{
	struct sockaddr_in *usin = (struct sockaddr_in *) uaddr;
	//struct rtable *rt;
	//int err;

	
	if (addr_len < sizeof(*usin)) 
	  	return(-EINVAL);

	/*
	 *	1003.1g - break association.
	 */
	 
	if (usin->sin_family==AF_UNSPEC)
	{
		sk->saddr=INADDR_ANY;
		sk->rcv_saddr=INADDR_ANY;
		sk->daddr=INADDR_ANY;
		sk->state = TCP_CLOSE;
#if 0
		if(uh_cache_sk == sk)
			uh_cache_sk = NULL;
#endif
		return 0;
	}

	if (usin->sin_family && usin->sin_family != AF_INET) 
	  	return(-EAFNOSUPPORT);

#if 0
	/* XXX */
	dst_release(xchg(&sk->dst_cache, NULL));

	err = ip_route_connect(&rt, usin->sin_addr.s_addr, sk->saddr,
			       sk->ip_tos|sk->localroute, sk->bound_dev_if);
	if (err)
		return err;
	if ((rt->rt_flags&RTCF_BROADCAST) && !sk->broadcast) {
		ip_rt_put(rt);
		return -EACCES;
	}
  	if(!sk->saddr)
	  	sk->saddr = rt->rt_src;		/* Update source address */
	if(!sk->rcv_saddr)
		sk->rcv_saddr = rt->rt_src;
	sk->daddr = rt->rt_dst;
	sk->dport = usin->sin_port;
	sk->state = TCP_ESTABLISHED;

	if(uh_cache_sk == sk)
		uh_cache_sk = NULL;

	sk->dst_cache = &rt->u.dst;
#endif
	return(0);
}


static void rt_udp_close(struct rtsocket *sk, long timeout)
{
	struct rtsocket *s;

	/* See for explanation: raw_close in ipv4/raw.c */
	sk->state = TCP_CLOSE;

	if(udp_sockets==sk){
		udp_sockets=sk->next;
	}else{
		for(s=udp_sockets;s;s=s->next){
			if(s->next==sk){
				s->next=sk->next;
				break;
			}
		}
	}
#if 0
	udp_v4_unhash(sk);
	sk->dead = 1;
	destroy_sock(sk);
#endif
}

static int rt_udp_queue_rcv_skb(struct rtsocket * sk, struct sk_buff *skb)
{
	/*
	 *	Charge it to the socket, dropping if the queue is full.
	 */

	if (skb_queue_len(&sk->incoming)>=20) {
		kfree_rtskb(skb);
		return -1;
	}
	__skb_queue_tail(&sk->incoming,skb);

	return 0;
}


static void rt_udp_deliver(struct rtsocket *sk, struct sk_buff *skb)
{
	rt_udp_queue_rcv_skb(sk,skb);

	if(sk->wakeup){
		sk->wakeup(sk->fd,sk->private);
	}
}


/*
 *	All we need to do is get the socket, and then do a checksum. 
 */
 
int rt_udp_rcv(struct rtsocket * sk,struct sk_buff *skb)
{
  	struct udphdr *uh;
	unsigned short ulen;
	//struct rtable *rt = (struct rtable*)skb->dst;
	u32 saddr = skb->nh.iph->saddr;
	u32 daddr = skb->nh.iph->daddr;

	/*
	 * First time through the loop.. Do all the setup stuff
	 * (including finding out the socket we go to etc)
	 */

	/*
	 *	Get the header.
	 */
	 
  	uh = skb->h.uh;
	__skb_pull(skb, skb->h.raw - skb->data);

#if 0
rt_printk("rt_udp_rcv() saddr=%08x daddr=%08x source=%04x dest=%04x\n",
	(int)saddr,(int)daddr,(int)uh->source,(int)uh->dest);
#endif

#if 0
  	ip_statistics.IpInDelivers++;
#endif

	/*
	 *	Validate the packet and the UDP length.
	 */
	 
	ulen = ntohs(uh->len);

#if 0
	if (ulen > len || ulen < sizeof(*uh)) {
		rt_printk(KERN_DEBUG "UDP: short packet: %d/%d\n", ulen, len);
		udp_statistics.UdpInErrors++;
		kfree_rtskb(skb);
		return(0);
	}
#endif
	skb_trim(skb, ulen);

	if (uh->check==0)
		skb->ip_summed = CHECKSUM_UNNECESSARY;
	else if (skb->ip_summed==CHECKSUM_HW) {
		if (udp_check(uh,ulen,saddr,daddr,skb->csum)) 
			goto csum_error;
		skb->ip_summed = CHECKSUM_UNNECESSARY;
	} else if (skb->ip_summed != CHECKSUM_UNNECESSARY)
		skb->csum = csum_tcpudp_nofold(saddr, daddr, ulen, IPPROTO_UDP, 0);

#if 0
	if(rt->rt_flags & (RTCF_BROADCAST|RTCF_MULTICAST))
		return udp_v4_mcast_deliver(skb, uh, saddr, daddr);
#endif

	sk = rt_udp_v4_lookup(saddr, uh->source, daddr, uh->dest, skb->dev->ifindex);
	
	if (sk == NULL) {
#if 0
  		udp_statistics.UdpNoPorts++;
#endif
#if 0
		rt_icmp_send(skb, ICMP_DEST_UNREACH, ICMP_PORT_UNREACH, 0);
#else
		rt_printk("UDP port unreachable\n");
#endif

		/*
		 * Hmm.  We got an UDP broadcast to a port to which we
		 * don't wanna listen.  Ignore it.
		 */
		kfree_rtskb(skb);
		return(0);
  	}
	rt_udp_deliver(sk, skb);
	return 0;

csum_error:
	/* 
	 * RFC1122: OK.  Discards the bad packet silently (as far as 
	 * the network is concerned, anyway) as per 4.1.3.4 (MUST). 
	 */
	rt_printk(KERN_DEBUG "UDP: bad checksum. From %d.%d.%d.%d:%d to %d.%d.%d.%d:%d ulen %d\n",
			NIPQUAD(saddr),
			ntohs(uh->source),
			NIPQUAD(daddr),
			ntohs(uh->dest),
			ulen);
#if 0
	udp_statistics.UdpInErrors++;
#endif
	kfree_rtskb(skb);
	return(0);
}


int rt_udp_add_socket(struct rtsocket *s)
{
	/* XXX should verify stuff */

	s->next=udp_sockets;
	udp_sockets=s;

	return 0;
}

int rt_udp_bind(struct rtsocket *s,struct sockaddr *uaddr,int addr_len)
{
	struct sockaddr_in *addr=(struct sockaddr_in *)uaddr;
	int chk_addr_ret;
	int snum;

rt_printk("rt_udp_bind()\n");
	if((s->state != TCP_CLOSE)			||
	   (addr_len < sizeof(struct sockaddr_in))
	   )
		return -EINVAL;

	chk_addr_ret = inet_addr_type(addr->sin_addr.s_addr);
	if(addr->sin_addr.s_addr != 0 && chk_addr_ret != RTN_LOCAL &&
	   chk_addr_ret != RTN_BROADCAST){
		return -EADDRNOTAVAIL;
	}

	s->rcv_saddr = s->saddr = addr->sin_addr.s_addr;
	if(chk_addr_ret == RTN_BROADCAST)
		s->saddr = 0;

	snum = ntohs(addr->sin_port);
	if(snum == 0)
		snum = rt_udp_good_socknum();
#if 0
	/* no <1024 port protection */
	if(snum < PROT_SOCK && !capable(CAP_NET_BIND_SERVICE))
		return -EACCES;
#endif

	s->num = snum;
	s->sport = htons(snum);
	s->daddr = 0;
	s->dport = 0;

	rt_udp_add_socket(s);
	
	return 0;
}

int rt_udp_listen(struct rtsocket *s,int backlog)
{
	/* XXX */

	return 0;
}

int rt_udp_recvmsg(struct rtsocket *s,struct msghdr *msg,int len)
{
	int error=0;
	int copied=0;
	struct sk_buff *skb;
	struct sockaddr_in *sin=msg->msg_name;

  	msg->msg_namelen=sizeof(*sin);

#if 0
	if (flags & MSG_ERRQUEUE)
		return ip_recv_error(s, msg, len);
#endif

	/* basically a copy of skb_recv_datagram: */

#if 0
	error = sock_error(s);
	if(error)
		goto no_packet;
#endif
#if 0
	if(s->shutdown & RCV_SHUTDOWN){
		error = 0;
		goto no_packet;
	}
#endif

#if 0
	if(connection_based(s) && s->state!=TCP_ESTABLISHED){
		error = -ENOTCONN;
		goto no_packet;
	}
#endif

	skb = __skb_dequeue(&s->incoming);

  	copied = skb->len - sizeof(struct udphdr);
	if (copied > len) {
		copied = len;
		msg->msg_flags |= MSG_TRUNC;
	}

	memcpy_tokerneliovec(msg->msg_iov,skb->h.raw+sizeof(struct udphdr),
			copied);

#if 0
	sk->stamp=skb->stamp;
#endif

	/* Copy the address. */
	if (sin)
	{
		sin->sin_family = AF_INET;
		sin->sin_port = skb->h.uh->source;
		sin->sin_addr.s_addr = skb->nh.iph->saddr;
  	}
#if 0
	if (sk->ip_cmsg_flags)
		ip_cmsg_recv(msg, skb);
#endif
	error = copied;
  
out_free:
  	//rtskb_free_datagram(s, skb);
	kfree_rtskb(skb);

out:
no_packet:
	return error;
}

struct rtsocket *rt_udp_socket_new(void)
{
	struct rtsocket *sock;

	sock=rtsocket_alloc();
	if(!sock)return sock;

	sock->state = TCP_CLOSE;
	sock->proto = IPPROTO_UDP;
	sock->ops=&rt_udp_socket_ops;
	sock->deliver=rt_udp_queue_rcv_skb;

	return sock;
}

struct rtsocket_ops rt_udp_socket_ops={
	bind:		rt_udp_bind,
	connect:	rt_udp_connect,
	listen:		rt_udp_listen,
	recvmsg:	rt_udp_recvmsg,
	sendmsg:	rt_udp_sendmsg,
	close:		rt_udp_close,
};


static struct rtsocket *rt_udp_socket;

__initfunc(void rt_udp_init(void))
{
	rt_udp_socket = rtsocket_alloc();
	rt_udp_socket->deliver = rt_udp_rcv;
	rt_udp_socket->proto = IPPROTO_UDP;

	rt_inet_add_protocol(rt_udp_socket);
}

